/*
******************************************************************************

   LibBUF: plain functions

   Copyright (C) 2001 Christoph Egger   [Christoph_Egger@t-online.de]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/


#include <ggi/internal/buf.h>
#include <ggi/internal/buf_debug.h>
#include <ggi/internal/galloc_ops.h>



int ggiBufClad(ggiBuf_t buf, int numbufs, ggiBuf_t *bufs, ggiBuf_t *res)
{
	return ggiBuf_clad(buf->vis, 0, numbufs, bufs, res);
}	/* ggiBufClad */

int ggiBufCladVis(ggi_visual_t vis, int numbufs, ggiBuf_t *bufs)
{
	return ggiBuf_clad(vis, BUF_CLAD_VISUAL, numbufs, bufs, NULL);
}	/* ggiBufCladVis */

int ggiBufSetGCForeground(ggiBuf_t buf, ggi_bufval val)
{
	if (buf == NULL) return GGI_EARGINVAL;

	buf->gc.fg_val = val;
	return 0;
}	/* ggiBufSetGCForeground */

int ggiBufSetGCBackground(ggiBuf_t buf, ggi_bufval val)
{
	if (buf == NULL) return GGI_EARGINVAL;

	buf->gc.bg_val = val;
	return 0;
}	/* ggiBufSetGCBackground */

int ggiBufSetGCFlags(ggiBuf_t buf, enum ggiBuf_flags flags)
{
	if (buf == NULL) return GGI_EARGINVAL;

	/* TODO: Check that desired flags have been approved by looking 
	 * at carb resource props.  Fail if they have not.
	 */
	buf->gc.flags = flags;
	return 0;
}	/* ggiBufGetGCForeground */

int ggiBufGetGCForeground(ggiBuf_t buf, ggi_bufval *val)
{
	if (buf == NULL) return GGI_EARGINVAL;

	val[0] = buf->gc.fg_val;
	return 0;
}	/* ggiBufGetGCForeground */

int ggiBufGetGCBackground(ggiBuf_t buf, ggi_bufval *val)
{
	if (buf == NULL) return GGI_EARGINVAL;

	val[0] = buf->gc.bg_val;
	return 0;
}	/* ggiBufGetGCForeground */

int ggiBufGetGCFlags(ggiBuf_t buf, enum ggiBuf_flags *flags)
{
	if (buf == NULL) return GGI_EARGINVAL;

	flags[0] = buf->gc.flags;
	return 0;
}	/* ggiBufGetGCForeground */

int ggiBufClearBuffer(ggiBuf_t buf)
{
	if (buf == NULL) return GGI_EARGINVAL;

	return ggiBufConst2Buf(buf,	/* dest */
			0,		/* xoffset */
			0,		/* yoffset */
			0,		/* stride */
			buf->gc.bg_val,	/* bg_val */
			buf->w, buf->h);
}	/* ggiBufClearBuffer */

int ggiBufGetValue(ggiBuf_t buf, size_t xoffset, size_t yoffset,
		   size_t stride, ggi_bufval *val)
{
	return ggiBufBuf2Mem(buf, xoffset, yoffset, stride,
			val, sizeof(*val), sizeof(*val), 1);
}	/* ggiBufGetValue */

int ggiBufPutValue(ggiBuf_t buf, size_t xoffset, size_t yoffset,
		   size_t stride, ggi_bufval val)
{
	return ggiBufMem2Buf(buf, xoffset, yoffset, stride,
			&val, sizeof(val), sizeof(val), 1);
}	/* ggiBufPutValue */
