/*
******************************************************************************

   Libbuf implementation for "stubs" target -- API functions.
  
   Copyright (c) Wed Aug 29 2001 by: 
	Brian S. Julin		bri@tull.umassp.edu
        Christoph Egger         Christoph_Egger@t-online.de
   
This software may be redistributed under the terms of the GNU Library
(a.k.a. "Lesser") Public License -- see the file COPYING in the parent 
directory of the source code.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.




******************************************************************************
*/

#include <stdio.h>
#include "stubsbuf.h"
#include <ggi/gg.h>


/* _All_ non-local (ie not declared 'static') functions and variables _must_
   be prefixed with the extension name, and for sublibs also with a unique
   sublib identifier. This is to keep the namespace clean on systems where
   all symbols are exported by default.
*/

/*-* API Implementation */

ggiBuf_t BUF_Stubs_ggiBuf_tie(ggi_visual_t vis, 
			      ggiGA_resource_handle handle_tank,
			      ggiGA_resource_handle handle_motor,
			      ggiGA_resource_handle handle_carb, 
			      int narg, void *args)
{
	char *fname;
	ggiBuf_t newbuf;
	ggi_ll_storage_t newstorage;
	struct ggiGA_resource_props *props;
	bufpriv *priv;

	priv = LIBGGI_BUFEXT(vis);

	/* Check is we actually own the resource we are being asked to tie */
	LIB_ASSERT(ggiGAGetVisual(handle_tank) == vis);
	if (ggiGAGetVisual(handle_tank) != vis) goto err0;

	/* TODO: Check some other stuff about the resource */

	props = ggiGAGetProperties(handle_tank);
	/* TODO: Check that we have code to deal with this type/subtype */

	newbuf = calloc(1, sizeof(struct ggiBuf));
	if (newbuf == NULL) goto err0;

	newstorage = calloc(1, sizeof(struct ggi_ll_storage));
	if (newstorage == NULL) goto err1;

	/* This all needs to be cleaned up nicely */
	if (!priv->def_ops.usage) {
		/* TODO: gcc: warning: assignment discards qualifiers from 
                 * pointer target type */
		fname = ggMatchConfig(priv->cfghand, "storage-stubs", NULL);
		if (fname == NULL) goto err2;
		ggiExtensionLoadDL(vis, fname, NULL,
				   &(priv->def_ops), "STORAGEdl_");
		free(fname);
	}	/* if */
	priv->def_ops.usage++;

	newbuf->storage_ops = &(priv->def_ops);
	newbuf->tank_p = ggiGAGetProperties(handle_tank);
	newbuf->carb_p = ggiGAGetProperties(handle_carb);
	newbuf->parts.res.tank = handle_tank;
	newbuf->parts.res.motor = handle_motor;
	newbuf->parts.res.carb = handle_carb;

	newbuf->type = ggiGAGetType(handle_tank);

	/* TODO: handle narg/qty or (size_t)*args/tank.buf_snap as per docs */
	newstorage->read =  newbuf->tank_p->sub.tank.read;
	newstorage->write = newbuf->tank_p->sub.tank.write;

#warning TODO: handle narg/qty or (size_t)*args/tank.buf_snap as per docs

	newbuf->w = newbuf->tank_p->size.area.x;
	newbuf->h = newbuf->tank_p->size.area.y;
	newstorage->stride = newbuf->w;

#warning TODO: use props->sub.tank.db

	newstorage->bitdepth = GT_SIZE(props->sub.tank.gt);
	newbuf->storage = newstorage;

	switch (newbuf->type) {
	case GA_RT_BUFFER_ZBUFFER:

		ggiBufSetGCFlags(newbuf, BUF_CMP_LTE);
		ggiBufSetGCForeground(newbuf, (1 << newstorage->bitdepth) - 1);
		ggiBufSetGCBackground(newbuf, 0);
		break;

	case GA_RT_BUFFER_ABUFFER:

		ggiBufSetGCFlags(newbuf, BUF_BLEND_OVER);
		ggiBufSetGCForeground(newbuf, (1 << newstorage->bitdepth) - 1);
		ggiBufSetGCBackground(newbuf, 0);
		break;

	default:
		goto err2;
	}	/* switch */

	return(newbuf);

 err2:
	free(newstorage);
 err1:
	free(newbuf);
 err0:
	BUFDPRINT("_tie returning NULL\n");
	return NULL;
}	/* BUF_Stubs_ggiBuf_tie */


int BUF_Stubs_ggiBuf_untie(ggi_visual *vis, ggiBuf_t *buf)
{
	BUFDPRINT("Function BUF_Stubs_ggiBuf_untie(ggi_visual *vis,
			ggiBuf_t *buf) called.\n");
	return(0);
}	/* BUF_Stubs_ggiBuf_untie */


int BUF_Stubs_getapi(ggi_visual_t vis,int num, char *apiname ,char *arguments)
{
	char zstr[4], astr[4], fstr[150], *ptr;
	bufpriv *priv;
	int rc;
	int flags = 0;

	priv = LIBGGI_BUFEXT(vis);
	zstr[0] = astr[0] = fstr[0] = '\0';

	/* We call libggi's original getapi function for this visual, and 
	 * try to do our best to replace any format strings with 
	 * libbuf-style renderers.
	 */
	rc = priv->getapi(vis, num, apiname, arguments);
	if (rc != 0) return rc;
	if (num == 0) return rc;

	apiname[255] = '\0'; /* color me me paranoid */	
	ptr = apiname + strlen(apiname);

	if (priv->achannel || (priv->abuf != NULL)) {
	  	if ((*(ptr - 2) == '-') || (*(ptr - 3) == '-')) {
			flags |= BUF_GETAPI_GIVE_PIXFMT;
		}	/* if */
		if (priv->abuf == NULL) flags |= BUF_GETAPI_ALPHA_USED;
		else sprintf(astr, "-a%d", priv->abuf->storage->bitdepth);
	}	/* if */
	if (priv->zbuf != NULL) {
		sprintf(zstr, "-z%d", priv->zbuf->storage->bitdepth);
	}	/* if */
	if (strstr(apiname, "-stubs") == NULL) {
		BUF_getapi_fstr(vis, fstr, flags);
	}	/* if */

	if (*(ptr - 2) == '-') ptr -= 2;
	if (*(ptr - 3) == '-') ptr -= 3;

	ptr += sprintf(ptr, "%s%s%s", fstr, astr, zstr);
	*ptr = '\0';
	return 0;

}	/* BUF_Stubs_getapi */


int BUF_Stubs_ggiBuf_clad(ggi_visual *vis, int flags, 
			  int numbufs, ggiBuf_t *bufs, ggiBuf_t *res)
{
	int err, idx, id;
	char sugname[256], args[256];
	bufpriv *priv;
	priv = LIBGGI_BUFEXT(vis);

	if (flags & BUF_CLAD_VISUAL) goto clad_vis;

	/* TODO: Clad other buffers */
	return GGI_EFATAL;

 clad_vis: 


	if (priv->is_clad) {
		BUFDPRINT("Visual is already clad, "
			  "must strip before cladding again.\n");
		return GGI_EFATAL;
	}
	priv->is_clad = 1;

	/* Save the current set of primitives. */
	if (priv->old_opdraw == NULL) {
		priv->old_opdraw = calloc(1, sizeof(struct ggi_visual_opdraw));
		if (priv->old_opdraw == NULL) return GGI_EFATAL;
	}	/* if */

	memcpy(priv->old_opdraw, vis->opdraw, sizeof(*vis->opdraw));

	for (idx = 0; idx < numbufs; idx++) {
		if (bufs[idx] == NULL) {
			/* This happens, when buffer allocation failed! */
			continue;
		}	/* if */

		if (bufs[idx]->type == GA_RT_BUFFER_ZBUFFER) {
			priv->zbuf = bufs[idx];
		}	/* if */

		if (bufs[idx]->type == GA_RT_BUFFER_ABUFFER) {
			if (0 /* TODO:test for alpha in pixel */) {
			} else {
				priv->abuf = bufs[idx];
			}	/* if */
		}	/* if */
	}	/* for */

	if ((priv->zbuf != NULL) || (priv->abuf != NULL)) {
		/* For now, until we have our own complete generic stubs,
		 * we need a copy of vanilla GGI generic-stubs.
		 */
		err = _ggiOpenDL(vis, "generic-stubs", NULL, NULL);
		if (err) {
			fprintf(stderr,"LibBuf: Can't open generic-stubs\n");
			return GGI_EFATAL;
		} else {
			BUFDPRINT("Success in loading generic stubs\n");
		}	/* if */
	}	/* if */

	/* Now we go get the rest of the api stack, with our special
	 * alpha/z/pixelformat in the suggest string courtesy of
	 * our special getapi implementation.  The first loop we
	 * do by hand for GGI opdraws, then the call to IndicateChange
	 * gets the extension operations. 
	 */
	for (id = 1; BUF_Stubs_getapi(vis,id,sugname,args) == 0; id++) {
		err = _ggiOpenDL(vis, sugname, args, NULL);
		if (err) {
			fprintf(stderr,"stubs-buf: Can't open the "
				"%s (%s) library.\n", sugname,args);
			if (id == 1) return GGI_EFATAL;	/* Must have stubs */
		} else {
			BUFDPRINT("Success in loading %s (%s)\n",sugname,args);
		}	/* if */
	}	/* for */
	ggiIndicateChange(vis, GGI_CHG_APILIST);
	
	return (0);
}	/* BUF_Stubs_ggiBuf_clad */


int BUF_Stubs_ggiBufStrip(ggi_visual *vis)
{
	bufpriv *priv;
	priv = LIBGGI_BUFEXT(vis);

	if (priv->old_opdraw != NULL) {

#warning TODO: Reload api stack fresh and use IndicateChange

		if (priv->is_clad) {
			/* Restore the visual's drawing primitives. */
			memcpy(vis->opdraw, priv->old_opdraw, 
			       sizeof(struct ggi_visual_opdraw));
		}	/* if */

		free(priv->old_opdraw);
		priv->old_opdraw = NULL;
	}       /* if */

	priv->abuf = NULL;
	priv->zbuf = NULL;
	priv->is_clad = 0;

	return GALLOC_OK;	
}	/* BUF_Stubs_ggiBufStrip */
