/*
******************************************************************************

   Private structures common to lowlevel libraries.

   Copyright (c) Fri Oct 5 2001 by: 
	Brian S. Julin		bri@tull.umassp.edu
        Christoph Egger         Christoph_Egger@t-online.de
  
   
This software may be redistributed under the terms of the GNU Library
(a.k.a. "Lesser") Public License -- see the file COPYING in the parent 
directory of the source code.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


******************************************************************************
*/

/*
 * NOTE: This file keeps separate some common attributes to internal
 * structures in all lowlevel libraries.  That way, although LibBuf
 * will be required to compile those libraries, they can be used 
 * without LibBuf itself being installed.  Only the storage sublib(s)
 * must be installed.
 */

#ifndef _GGI_LOWLEVEL_H
#define _GGI_LOWLEVEL_H

#include <ggi/internal/internal.h>
#include <ggi/internal/structs.h>
#include <ggi/galloc.h>
#include <ggi/ga_prop.h>
#include <string.h>

/* 
 * Storage Access.  Note, storage libraries do *not* do bounds checking.
 * This keeps them, and their associated structures leightweight and
 * minimalistic.
 */
struct ggi_ll_storage {
	int bitdepth, stride; 
	void *read, *write;
	void *priv;		/* Usually this will point to the owning
				 * ggiBuf_t/ggiOvl_t/ggiBlt_source_t,
				 * and their priv structure used instead.
				 */
	ggi_visual_t vis;
};
typedef struct ggi_ll_storage *ggi_ll_storage_t;

typedef ggi_pixel ggi_storageval;

struct ggi_ll_storage_ops;
typedef struct ggi_ll_storage_ops *ggi_ll_storage_ops_t;

typedef int (storage2mem_func)(ggi_ll_storage_t src, size_t xoffset,
			       size_t yoffset, size_t stride,
			       void *dest, size_t deststride,
			       size_t linewidth, size_t numlines);

typedef int (mem2storage_func)(ggi_ll_storage_t dest, size_t xoffset,
			       size_t yoffset, size_t stride,
			       void *membuf, size_t srcstride,
			       size_t linewidth, size_t numlines);

typedef int (storage2storage_func)(ggi_ll_storage_t dest, size_t xoffset,
				   size_t yoffset, size_t stride,
				   ggi_ll_storage_t src, 
				   ggi_ll_storage_ops_t srcops,
				   size_t srcxoffset,
				   size_t srcyoffset, size_t srcstride,
				   size_t linewidth, size_t numlines);

typedef int (const2storage_func)(ggi_ll_storage_t dest, size_t xoffset,
				 size_t yoffset, size_t stride,
				 ggi_storageval val, size_t linewidth, 
				 size_t numlines);

#define GGI_ACCESSFUNC_UNNEEDED	0x01
#define GGI_ACCESSFUNC_NOFLUSH	0x02

struct ggi_ll_storage_ops {
        int 			usage;

	storage2mem_func	*storage2mem;
	mem2storage_func	*mem2storage;
	storage2storage_func	*storage2storage;
	const2storage_func	*const2storage;
};


#endif /* _GGI_LOWLEVEL_H */


